/*

nsisPatchGen
Copyright (C) 2006  Vibration Technology Ltd. ( http://www.vibtech.co.uk )

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

/**
  * \file
**/

#ifndef DIRECTORY_DIFF_H
#define DIRECTORY_DIFF_H
#pragma once

#include "StartExternals.h"
#include <QStringList>
#include <QDir>
#include "StopExternals.h"

class QString;
class DirectoryDiffTest;

/// Runs a recursive diff through a direcory hierarchy.
/// Compiles a list of new, modified and deleted files.
class DirectoryDiff
{
public:
	DirectoryDiff( );
	virtual ~DirectoryDiff( );

	void compare( const QString& directory1, const QString& directory2 ) throw( QString );

	const QStringList& filesAdded() const;
	const QStringList& filesRemoved() const;
	const QStringList& filesModified() const;

	const QStringList& directoriesAdded() const;
	const QStringList& directoriesRemoved() const;

	void includeSystem( bool enable );
	void includeHidden( bool enable );

private:

	/// Private version of compare method. 
	/// Can be called recursively as we drill down through sub-directories.
	void comparePrivate( const QString& directory1, const QString& directory2 ) throw( QString );


	/// Compares the sub-directories in directory 1 & 2
	void compareSubDirectoriesIn( const QString& directory1, const QString& directory2 );

	/// Compares the files in directories 1 & 2
	void compareFilesIn( const QString& directory1, const QString& directory2 );

	/// Compares two files.
	void compareFile( const QString& filename1, const QString& filename2 );

	/// Handles a directory that has been removed.
	/// Add all the files to the deleted list.
	void directoryRemoved( const QString& directory1 );

	/// Handles a directory that has been created.
	/// Add all the files to the new list.
	void directoryAdded( const QString& directory2 );

	void addToList( const QString& label, QStringList& list, const QStringList& items );

	void addToList( const QString& label, QStringList& list, const QString& item );

	void log( const QString& label, const QStringList& list );

	void log( const QString& label, const QString& item );

	// Check the ignore mask to prevent excess work
	bool isIgnored( QString path );

	QString mDirectory1;
	QString mDirectory2;

	QStringList mFilesAdded;
	QStringList mFilesRemoved;
	QStringList mFilesModified;

	QStringList mDirectoriesAdded;
	QStringList mDirectoriesRemoved;

	QList<QRegExp> mIgnoreList;
	QRegExp mBasePathRemover;

	QDir::Filters mEntryListFilter;

	friend class DirectoryDiffTest;

};

#endif // DIRECTORY_DIFF_H