/*

nsisPatchGen
Copyright (C) 2006  Vibration Technology Ltd. ( http://www.vibtech.co.uk )

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

/**
  * \file
**/

#include "StartExternals.h"
#include <QString>
#include <QStringList>
#include <QProcess>
#include <QDir>
#include "StopExternals.h"

#include "PatchGenerator.h"

PatchGenerator::PatchGenerator( )
{
}

PatchGenerator::~PatchGenerator( )
{
}

void PatchGenerator::createPatchFile( const QString& source, 
									  const QString& destination, 
									  const QString& patchFile ) throw( QString )
{
	static const QString REPLACE_PATCH_FILE("-R");

	QStringList arguments;
	arguments.append( REPLACE_PATCH_FILE );
	arguments.append( source );
	arguments.append( destination );
	arguments.append( patchFile );

	const QString GENPAT_EXE( "GenPat.exe" );

	qDebug( "Creating patch file: %s", patchFile );

	createDirectoryForPatchFileIfNecessary( patchFile );

	QProcess genPat;
	genPat.start( GENPAT_EXE, arguments );

	if( genPat.error() == QProcess::FailedToStart ||
		genPat.error() == QProcess::Crashed )
	{
		throw QString( "GenPat failed to start" );
	}

	const int MSECS_IN_MIN = 60 * 1000;
	const int TIMEOUT_SECS = 60 * MSECS_IN_MIN; // 60 mins

	if( ! genPat.waitForFinished( TIMEOUT_SECS ) )
	{
		throw( QString( "Timed out waiting on GenPat") );
	}

	switch( genPat.exitCode( ) )
	{
	case 0:
		// success!!!
		break;	

	case 1:
		{
			QString args = arguments.join(" ");
			throw QString( "INTERNAL ERROR: GenPat arguments missing: " + GENPAT_EXE + " " + args  );
		}
		break;

	case 2:
		throw QString( "Other GenPat error" );
		break;

	case 3:
		throw QString( "GenPat: Source file already has a patch in specified patch file (=error)" );
		break;

	default:
		throw QString( "GenPat: undefined error" );
		break;
	}

}

bool PatchGenerator::createDirectoryForPatchFileIfNecessary( const QString& patchFile )
{
	QString path = patchFile;	
	if( patchFile.contains("\\") )
	{
		path.truncate( path.lastIndexOf("\\") );
	}

	QDir patchDir( path );

	QDir localDir;
	QString dir = localDir.dirName();

	if( !patchDir.exists() )
	{
		QDir localDir;
		QString dir = localDir.dirName();
		if( !localDir.mkpath( path ) )
		{
			return false;
		}
	}

	return true;
}


