/*

nsisPatchGen
Copyright (C) 2006  Vibration Technology Ltd. ( http://www.vibtech.co.uk )

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/
#include "StartExternals.h"
#include <QString>
#include <QStringList>
#include <QCoreApplication>
#include <QTextStream>
#include <QtDebug>
#include "StopExternals.h"

#include "DirectoryDiff.h"
#include "NsisGenerator.h"
#include "PatchGenerator.h"

const QString NSIS_VERSION = "1.0.2";

// forward declaration
void usage( QString appName );

int main(int argc, char *argv[])
{
    QCoreApplication app(argc, argv);
	QString APP_ID = "\r\nNSIS Patch Generation Utility v" + NSIS_VERSION + "\r\n";
	qDebug( "%s", APP_ID );
	
	DirectoryDiff directoryDiff;

	QStringList args = app.arguments();

	if( args.contains( "--help" ) || args.contains( "-h") )
	{
		usage( args[0] );
	}

	if( args.contains( "--hidden" ) )
	{
		directoryDiff.includeHidden( true );
		args.removeAll( "--hidden" );
	}

	if( args.contains( "--system" ) )
	{
		directoryDiff.includeSystem( true );
		args.removeAll( "--system" );
	}

	QString appName	= args[0].replace("/", "\\");	// task name - as typed in.
	QString directory1;								// mandatory
	QString directory2;								// mandatory
	QString patchDirectory("patchFiles");			// optional
	QString outputFile( "patchFiles.nsi" );			// optional

	if ( args.count() > 2 )
	{
		directory1 = args[1].replace("/", "\\");
		directory2 = args[2].replace("/", "\\");

		if( args.size() > 3 )
			patchDirectory = args[3].replace("/", "\\");
		if( args.size() > 4 )
			outputFile = args[4].replace("/", "\\");
	}
	else
	{
		QFile settings( "settings.txt" );
		if ( settings.open( QIODevice::ReadOnly | QIODevice::Text ) )
		{
			args.clear();

			QTextStream in( &settings );
			while ( !in.atEnd() )
				args.append( in.readLine() );

			if( args.contains( "--hidden" ) )
			{
				directoryDiff.includeHidden( true );
				args.removeAll( "--hidden" );
			}

			if( args.contains( "--system" ) )
			{
				directoryDiff.includeSystem( true );
				args.removeAll( "--system" );
			}

			if ( args.count() > 1 )
			{
				directory1 = args[0].replace("/", "\\");
				directory2 = args[1].replace("/", "\\");

				if( args.size() > 2 )
					patchDirectory = args[2].replace("/", "\\");
				if( args.size() > 3 )
					outputFile = args[3].replace("/", "\\");
			}
			else
			{
				usage( appName );
			}
		}
		else
		{
			usage( appName );
		}
	}

	try
	{
		directoryDiff.compare( directory1, directory2 );

		PatchGenerator patchGenerator;
		foreach( QString fileName, directoryDiff.filesModified() )
		{
			patchGenerator.createPatchFile( directory1 + "\\" + fileName, 
											directory2 + "\\" + fileName, 
											patchDirectory + "\\" + fileName + ".pat" );
		}

		NsisGenerator nsisGenerator( outputFile );
		nsisGenerator.open();
		nsisGenerator.directoriesAdded( directoryDiff.directoriesAdded() );
		nsisGenerator.directoriesRemoved( directoryDiff.directoriesRemoved() );
		nsisGenerator.filesAdded( directoryDiff.filesAdded() );
		nsisGenerator.filesRemoved( directoryDiff.filesRemoved() );
		nsisGenerator.filesModified( directoryDiff.filesModified() );
		nsisGenerator.close();

		qDebug() << "Created NSIS patch installer file: " << outputFile;
	}
	catch( QString exception )
	{
		qCritical() << appName << ": " << exception;
	}

    return 0;
}


void usage( QString appName )
{
	qCritical() << "Usage: " << appName << " [--hidden] [--system] directory1 directory2 [patch output directory] [NSIS output file]" << endl;
	qCritical() << "--hidden              : include hidden files in comparison";
	qCritical() << "--system              : include system files in comparison";
	qCritical() << "directory1            : the top level directory containing the original set of files";
	qCritical() << "directory2            : top level directory containing the target set of files";
	qCritical() << "patch output direcory : directory into which the patch files will be placed (default \".\\patchFiles\")";
	qCritical() << "NSIS output file      : name of the generated NSIS file (default \"patchFiles.nsi\")";
	qCritical() << "OPTIONAL: You can create a settings.txt with these options listed per line";
	exit( -1 );
}
