/*

nsisPatchGen
Copyright (C) 2006  Vibration Technology Ltd. ( http://www.vibtech.co.uk )

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "StartExternals.h"
#include <QFile>
#include <QStringList>
#include <QByteArray>
#include "StopExternals.h"

#include "NsisGeneratorTest.h"
#include "NsisGenerator.h"


// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( NsisGeneratorTest );

const QString OUTPUT_PATH("testData\\NsisGeneratorTest\\output\\");
const QString EXPECTED_PATH("testData\\NsisGeneratorTest\\expected\\");

void NsisGeneratorTest::setUp()
{
}

void NsisGeneratorTest::tearDown()
{
}

void NsisGeneratorTest::testConstructor()
{
	NsisGenerator ng( OUTPUT_PATH + "aFileName.txt" );
}

void NsisGeneratorTest::testFilesAdded()
{
	NsisGenerator ng( OUTPUT_PATH + "testFilesAdded.nsi" );

	try
	{
		ng.open();
	}
	catch( QString error )
	{
		qDebug() << "Caught unexpected exception from NsisGenerator::open " << error << endl;
		CPPUNIT_ASSERT( false );
	}

	QStringList files;
	files.append( "file1.txt" );
	files.append( "file2.txt" );
	files.append( "directory1\\file3.txt" );
	files.append( "directory1\\directory2\\file4.txt" );

	ng.filesAdded( files );

	ng.close();

	QFile nsisFile( OUTPUT_PATH + "testFilesAdded.nsi" );
	CPPUNIT_ASSERT( nsisFile.open( QIODevice::ReadOnly ) );
	CPPUNIT_ASSERT( nsisFile.size() > 0 );

	QFile expectedFile( EXPECTED_PATH + "testFilesAdded.nsi" );
	CPPUNIT_ASSERT( expectedFile.open( QIODevice::ReadOnly ) );

	CPPUNIT_ASSERT( nsisFile.readAll() == expectedFile.readAll() );
}



void NsisGeneratorTest::testFilesRemoved()
{
	NsisGenerator ng( OUTPUT_PATH + "testFilesRemoved.nsi" );
	try
	{
		ng.open();
	}
	catch( QString error )
	{
		qDebug() << "Caught unexpected exception from NsisGenerator::open " << error << endl;
		CPPUNIT_ASSERT( false );
	}

	QStringList files;
	files.append( "file1.txt" );
	files.append( "file2.txt" );
	files.append( "directory1\\file3.txt" );
	files.append( "directory1\\directory2\\file4.txt" );

	ng.filesRemoved( files );
	ng.close();

	QFile nsisFile( OUTPUT_PATH + "testFilesRemoved.nsi" );
	CPPUNIT_ASSERT( nsisFile.open( QIODevice::ReadOnly ) );
	CPPUNIT_ASSERT( nsisFile.size() > 0 );

	QFile expectedFile( EXPECTED_PATH + "testFilesRemoved.nsi" );
	CPPUNIT_ASSERT( expectedFile.open( QIODevice::ReadOnly ) );

	CPPUNIT_ASSERT( nsisFile.readAll() == expectedFile.readAll() );
}


void NsisGeneratorTest::testFilesModified()
{
	NsisGenerator ng( OUTPUT_PATH + "testFilesModified.nsi" );
	try
	{
		ng.open();
	}
	catch( QString error )
	{
		qDebug() << "Caught unexpected exception from NsisGenerator::open " << error << endl;
		CPPUNIT_ASSERT( false );
	}

	QStringList files;
	files.append( "file1.txt" );
	files.append( "file2.txt" );
	files.append( "directory1\\file3.txt" );
	files.append( "directory1\\directory2\\file4.txt" );

	ng.filesModified( files );
	ng.close();

	QFile nsisFile( OUTPUT_PATH + "testFilesModified.nsi" );
	CPPUNIT_ASSERT( nsisFile.open( QIODevice::ReadOnly ) );
	CPPUNIT_ASSERT( nsisFile.size() > 0 );

	QFile expectedFile( EXPECTED_PATH + "testFilesModified.nsi" );
	CPPUNIT_ASSERT( expectedFile.open( QIODevice::ReadOnly ) );

	CPPUNIT_ASSERT( nsisFile.readAll() == expectedFile.readAll() );
}


void NsisGeneratorTest::testDirectoriesAdded()
{
	NsisGenerator ng( OUTPUT_PATH + "testDirectoriesAdded.nsi" );
	try
	{
		ng.open();
	}
	catch( QString error )
	{
		qDebug() << "Caught unexpected exception from NsisGenerator::open " << error << endl;
		CPPUNIT_ASSERT( false );
	}

	QStringList directories;
	directories.append( "dir1" );
	directories.append( "dir2" );
	directories.append( "dir2\\dir3" );
	directories.append( "dir2\\dir3\\dir4" );

	ng.directoriesAdded( directories );
	ng.close();

	QFile nsisFile( OUTPUT_PATH + "testDirectoriesAdded.nsi" );
	CPPUNIT_ASSERT( nsisFile.open( QIODevice::ReadOnly ) );
	CPPUNIT_ASSERT( nsisFile.size() > 0 );

	QFile expectedFile( EXPECTED_PATH + "testDirectoriesAdded.nsi" );
	CPPUNIT_ASSERT( expectedFile.open( QIODevice::ReadOnly ) );

	CPPUNIT_ASSERT( nsisFile.readAll() == expectedFile.readAll() );
}


void NsisGeneratorTest::testDirectoriesRemoved()
{
	NsisGenerator ng( OUTPUT_PATH + "testDirectoriesRemoved.nsi" );
	try
	{
		ng.open();
	}
	catch( QString error )
	{
		qDebug() << "Caught unexpected exception from NsisGenerator::open " << error << endl;
		CPPUNIT_ASSERT( false );
	}

	QStringList directories;
	directories.append( "dir1" );
	directories.append( "dir2" );
	directories.append( "dir2\\dir3" );
	directories.append( "dir2\\dir3\\dir4" );

	ng.directoriesRemoved( directories );
	ng.close();

	QFile nsisFile( OUTPUT_PATH + "testDirectoriesRemoved.nsi" );
	CPPUNIT_ASSERT( nsisFile.open( QIODevice::ReadOnly ) );
	CPPUNIT_ASSERT( nsisFile.size() > 0 );

	QFile expectedFile( EXPECTED_PATH + "testDirectoriesRemoved.nsi" );
	CPPUNIT_ASSERT( expectedFile.open( QIODevice::ReadOnly ) );

	CPPUNIT_ASSERT( nsisFile.readAll() == expectedFile.readAll() );
}

