/*

nsisPatchGen
Copyright (C) 2006  Vibration Technology Ltd. ( http://www.vibtech.co.uk )

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "StartExternals.h"
#include <QFile>
#include <QDir>
#include "StopExternals.h"

#include "PatchGeneratorTest.h"
#include "PatchGenerator.h"


// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( PatchGeneratorTest );

const QString ROOT_PATCH_FILE_DIR( "testData\\PatchGeneratorTest\\output" );


void PatchGeneratorTest::setUp()
{
}

void PatchGeneratorTest::tearDown()
{
}

void PatchGeneratorTest::testConstructor()
{
	PatchGenerator pg;
}

void PatchGeneratorTest::testCreatePatchFile()
{
	PatchGenerator pg;

	// current directory is "src/test"
	const QString SOURCE("testData\\PatchGeneratorTest\\testCreatePatchFile1.txt");
	const QString DESTINATION("testData\\PatchGeneratorTest\\testCreatePatchFile2.txt");
	const QString PATCH_FILE( ROOT_PATCH_FILE_DIR + "\\directory1\\testCreatePatchFile.txt.pat" );

	try 
	{
		pg.createPatchFile( SOURCE, DESTINATION, PATCH_FILE );
	}
	catch( QString error )
	{
		qDebug() << "Caught unexpected exception: " << error;
		CPPUNIT_ASSERT( false );
	}

	QFile patchFile( PATCH_FILE );
	CPPUNIT_ASSERT( patchFile.exists() );
	CPPUNIT_ASSERT( patchFile.open( QIODevice::ReadOnly ) );
	CPPUNIT_ASSERT( !patchFile.readAll().isEmpty() );

}

void PatchGeneratorTest::testCreatePatchFileFailures()
{
	PatchGenerator pg;

	const QString SOURCE("testData\\PatchGeneratorTest\\testCreatePatchFile1.txt");
	const QString INVALID_SOURCE("testData\\PatchGeneratorTest\\invalidFile1.txt");

	const QString DESTINATION("testData\\PatchGeneratorTest\\testCreatePatchFile2.txt");
	const QString INVALID_DESTINATION("testData\\PatchGeneratorTest\\invalidFile2.txt");

	const QString PATCH_FILE( ROOT_PATCH_FILE_DIR + "\\directory2\\testCreatePatchFile.txt.pat");

	QFile patchFile( PATCH_FILE );
	if( patchFile.exists() )
	{
		patchFile.remove();
	}

	bool exceptionThrown = false;

	try
	{
		pg.createPatchFile( INVALID_SOURCE, DESTINATION, PATCH_FILE );
		exceptionThrown = false;
	}
	catch( QString error )
	{
		exceptionThrown = true;
	}
	CPPUNIT_ASSERT( exceptionThrown );
	CPPUNIT_ASSERT( ! patchFile.exists() );



	try
	{
		pg.createPatchFile( SOURCE, INVALID_DESTINATION, PATCH_FILE );
		exceptionThrown = false;
	}
	catch( QString error )
	{
		exceptionThrown = true;
	}
	CPPUNIT_ASSERT( exceptionThrown );
	CPPUNIT_ASSERT( ! patchFile.exists() );
}


